/**
 * Created by zdm on 2019/5/7.
 */

import java.io.*;
import java.util.ArrayList;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPClientConfig;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;

/**
 * 列出FTP服务器上指定目录下面的所有文件
 */
public class FTPListAllFiles {
    private static Logger logger = Logger.getLogger(FTPListAllFiles.class);
    public static FTPClient ftp = new FTPClient();
    public ArrayList<String> arFiles;
    private static final String FAR_SERVER_URL = "172.26.0.220";
    private static final int SERVER_PORT = 21;
    private static final String SERVER_USER = "Telecom";
    private static final String SERVER_PWD = "telecom@jkzl2019";
    private static final String path = "/data/rec2/2019/0507/08/";
    private static final String showPath = "D:\\testFile\\";
    @Value("${file.default_path}")
    private String fileDefaultPath;


    /**
     * 登陆FTP服务器
     *
     * @param host     FTPServer IP地址
     * @param port     FTPServer 端口
     * @param username FTPServer 登陆用户名
     * @param password FTPServer 登陆密码
     * @return 是否登录成功
     * @throws IOException
     */
    public static boolean login(String host, int port, String username, String password) throws IOException {
        ftp.connect(host, port);
        if (FTPReply.isPositiveCompletion(ftp.getReplyCode())) {
            if (ftp.login(username, password)) {
                ftp.setControlEncoding("GBK");
                ftp.setFileType(FTPClient.BINARY_FILE_TYPE);
                //设置linux环境
                FTPClientConfig conf = new FTPClientConfig( FTPClientConfig.SYST_UNIX);
                ftp.configure(conf);
                return true;
            }
        }
        if (ftp.isConnected()) {
            ftp.disconnect();
        }
        return false;
    }

    /**
     * 关闭数据链接
     *
     * @throws IOException
     */
    public void disConnection() throws IOException {
        if (this.ftp.isConnected()) {
            this.ftp.disconnect();
        }
    }

    /**
     * 递归遍历出目录下面所有文件
     *
     * @param pathName 需要遍历的目录，必须以"/"开始和结束
     * @throws IOException
     */
    public static String List(String pathName) throws IOException {
        StringBuffer filename = new StringBuffer();
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //更换目录到当前目录
            ftp.changeWorkingDirectory(path);
            //设置访问被动模式
            ftp.enterLocalPassiveMode();
            ftp.setRemoteVerificationEnabled(false);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
                FTPFile[] files = ftp.listFiles();
                if (files != null) {
                    for (int i = 0; i < files.length; i++) {
                        if (files[i].isFile()) {
                            downloadFileFromFTP(ftp, path+files[i].getName(), showPath,files[i].getName());
                        }
                }
            }
        }
        return filename.toString();
    }


    /**
     * 从FTP下载文件到本地
     * @param ftpClient     已经登陆成功的FTPClient
     * @param ftpFilePath   FTP上的目标文件路径
     * @param localFilePath 下载到本地的文件路径
     */
    public static void downloadFileFromFTP(FTPClient ftpClient, String ftpFilePath, String localFilePath,String fileName) {
        InputStream is = null;
        FileOutputStream fos = null;
        try {
            // 获取ftp上的文件
            is = ftpClient.retrieveFileStream(ftpFilePath);
            fos = new FileOutputStream(new File(localFilePath+fileName));
            // 文件读取方式一
            int i;
            byte[] bytes = new byte[1024];
            while ((i = is.read(bytes)) != -1) {
                fos.write(bytes, 0, i);
            }
            ftpClient.completePendingCommand();
            logger.info("FTP文件下载成功！");
        } catch (Exception e) {
            logger.error("FTP文件下载失败！", e);
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }


    //测试
    public static void main(String[] args) throws IOException {
        login("172.26.0.220",21,"Telecom","telecom@jkzl2019");
        List(path);
    }
}
